###############################################################
# 
# Extendable Functional Mapping Application(FunMap2)
#
# report utility
#    1). fre.report_dat   
#    2). fre.report_res   
#    3). fre.report_res2   
#
# History:
# 12/15/2011 Version 1.1
#
###############################################################

frep.report_dat<-function( dat ) 
{
	Report.AddParaLine( tab.pos=c(300, 340, -1 ), tab.align=c("R", "C", "L"), c("Model:", "", dat$model ) );
	Report.AddParaLine( tab.pos=c(300, 340, -1 ), tab.align=c("R", "C", "L"), c("Pheno. file:", "", dat$pheno_file ) );
	Report.AddParaLine( tab.pos=c(300, 340, -1 ), tab.align=c("R", "C", "L"), c("Geno. file:",  "", dat$geno_file ) );
	Report.AddParaLine( tab.pos=c(300, 340, -1 ), tab.align=c("R", "C", "L"), c("Sample size:", "", dat$sample_n) );
	Report.AddParaLine( tab.pos=c(300, 340, -1 ), tab.align=c("R", "C", "L"), c("Sample times:","", length(dat$sample_times) ) );
	Report.AddParaLine( tab.pos=c(300, 340, -1 ), tab.align=c("R", "C", "L"), c("SNP count:","", length(dat$raw_snps[,1]) ) );
	Report.AddParaLine( tab.pos=c(300, 340, -1 ), tab.align=c("R", "C", "L"), c("Non-rare count:","", length(dat$snps[,1]) ) );

	Report.AddParaBreak();
	
	if (dat$sample_times>1)
	{
		c1 <- call("fplot.tiled_curves", dat$pheno_y, dat$covar_z );
		Report.AddFigure( c1, " ", c(6, 6)*254, left.margin=0.5*254);
	}

	if (dat$sample_times>1)
	{
		c2 <- call("fplot.overlapping_curves", dat$pheno_y, dat$covar_z  );
		Report.AddFigure( c2, " ", c(6, 6)*254, left.margin=0.5*254);
	}

	Report.AddParaBreak();

	invisible();
} 

frep.report_pv_profile<-function( dat, res ) 
{
	pvs<- cbind( res$snp_chrs, res$LR2.res[,3] );
	rownames(pvs) <- res$snp_names;

	c1 <- call("fplot.manhattan", pvs );

	Report.AddFigure( c1, "p-value Profile", c(6, 3)*254, left.margin=0.5*254);
	Report.AddParaBreak();

	invisible();
} 

frep.report_sig_curve<-function( snp.par, snp.n) 
{

	for (i in 1:snp.n)
	{
		c2 <- call("fplot.single_curve", 
				snp_name = snp.par[[i]]$snp_name,
				QQ       = snp.par[[i]]$QQ,
				Qq       = snp.par[[i]]$Qq,
				qq       = snp.par[[i]]$qq,
				add      = snp.par[[i]]$add,
				dom      = snp.par[[i]]$dom );

		Report.AddFigure( c2, "", c(3, 2.25)*254, left.margin=0.25*254);

		if (i%%2==0)
			Report.AddParaBreak(50);
	}

	if (snp.n%%2==1)
	{
		c2 <- call("fplot.blank_rect");
		Report.AddFigure( c2, "", c(3, 2.25)*254, left.margin=0.25*254);
	}
	
	Report.AddParaBreak();
	invisible();
}

frep.report_adh2_profile<-function( pdata, data.cnt )
{
	n.col <- dim(pdata)[2];

	if ( data.cnt==3)
	{
		c1 <- call("fplot.single_adh2", 
					pdata    = pdata[, n.col-2, drop=F],
					chr      = pdata[, 1, drop=F ],
					ytitle   = "Estimated additive effect");

		Report.AddFigure( c1, "", c(6, 3)*254, left.margin=0.5*254);

		c2 <- call("fplot.single_adh2", 
					pdata    = pdata[, n.col-1, drop=F],
					chr      = pdata[, 1, drop=F ],
					ytitle   = "Estimated dominant effect");

		Report.AddFigure( c2, "", c(6, 3)*254, left.margin=0.5*254);

		c3 <- call("fplot.single_adh2", 
					pdata    = pdata[, n.col, drop=F],
					chr      = pdata[, 1, drop=F ],
					ytitle   = "Heritability");

		Report.AddFigure( c3, "", c(6, 3)*254, left.margin=0.5*254);
	}

	if ( data.cnt==2)
	{
		c1 <- call("fplot.single_adh2", 
					pdata    = pdata[, n.col-1, drop=F],
					chr      = pdata[, 1, drop=F ],
					ytitle   = "Estimated additive effect");

		Report.AddFigure( c1, "", c(6, 3)*254, left.margin=0.5*254);

		c2 <- call("fplot.single_adh2", 
					pdata    = pdata[, n.col-0, drop=F],
					chr      = pdata[, 1, drop=F ],
					ytitle   = "Estimated dominant effect");

		Report.AddFigure( c2, "", c(6, 3)*254, left.margin=0.5*254);
	}

	Report.AddParaBreak();

	invisible();
}