###############################################################
# 
# System management utility
#
# History:
# 03/17/2010 Version 0.1
#
###############################################################

#--------------------------------------------------------------
# Statgen.sys
#
#--------------------------------------------------------------
Statgen.sys<-function( module )
{
	i_module    <- module
	i_seq_id    <- 0
	i_start_timer<-proc.time()
	i_hash      <- NULL
	i_task.level<-0;
	i_task.stack<- array(0, dim=c(1000, 4));

	get_seq_id<-function()
	{
		i_seq_id <<- i_seq_id+1;
		return (i_seq_id);
	}
	
	get_tmpfile<-function(prefix=NULL, suffix=NULL)
	{
		if (is.null(prefix)) prefix<-"LStmp";
		if (is.null(suffix)) suffix<-".dat";
		
		f.name <- paste(prefix, get_seq_id(),suffix, sep="");
		return(f.name);
	}

	reset<-function()
	{
		i_seq_id <<- 0;
		i_start_timer<<-0;
		task.reset();

		i_hash <<- new.env(hash=TRUE, parent=emptyenv(), size=100L);
		
		assign("plot_doctype",  "pdf",   i_hash);
	}

	task.reset<- function()
	{
		i_task.level <<- 0;
	}

	task.push<-function( loop.st, loop.end )
	{
		if (i_task.level>=1000)
			stop("The level of task stack is too big.");

		i_task.level <<- i_task.level+1;	
		i_task.stack[ i_task.level, 1 ] <<- loop.st; 
		i_task.stack[ i_task.level, 2 ] <<- loop.end; 
		i_task.stack[ i_task.level, 3 ] <<- loop.st; 
		if (i_task.level==1)
			i_start_timer <<- proc.time();
		i_task.stack[ i_task.level, 4 ] <<- proc.time()[3]; 
	}

	task.pop<-function()
	{
		if (i_task.level<1)
			stop("The level of task stack is too small<1.");

		i_task.level <<- i_task.level-1;	

	}

	task.enter<-function(loop)
	{
		i_task.stack[ i_task.level, 3 ] <<- loop; 
	}

	task.get<-function(loop)
	{
		i_task.stack[ i_task.level, 3 ] <<- loop; 
		
		per.per <- 0;
		per.nLast<-1;
		per.cum <- 0;

		for(i in 1:i_task.level)
		{
			per.per <- per.nLast/( 1 + i_task.stack[ i, 2 ] - i_task.stack[ i, 1 ]);
			per.cum <- per.cum + per.per*(i_task.stack[ i, 3 ]-i_task.stack[ i, 1 ]);

			per.nLast <- per.per;
		}
		

		sTime1<-"";
		sTime2<-"";
		if (per.cum!=0)
		{
			nt <- proc.time() - i_start_timer;
			sys_t <-  round(c(nt[3], nt[3]*( 1 - per.cum)/per.cum ));

			if (sys_t[1]>60)
				sTime1 <- sprintf( "%02g:%02g:%02g", sys_t[1]%/%3600, (sys_t[1]- sys_t[1]%/%3600*3600)%/%60, sys_t[1]%%60 )
			else
				sTime1 <- sprintf( "%02g seconds", sys_t[1] );
			if (sys_t[2]>60)
				sTime2 <- sprintf( "%02g:%02g:%02g", sys_t[2]%/%3600, (sys_t[2]- sys_t[2]%/%3600*3600)%/%60, sys_t[2]%%60 )
			else
				sTime2 <- sprintf( "%02g seconds", sys_t[2] );
		}

		return(list(per=per.cum*100, elapse = sTime1, left=sTime2));
	}


	task_start<-function(loop.st, loop.end, msg=NULL)
	{
		task.push( loop.st, loop.end );

		if ( !is.null(msg) )
		{
			cat(msg, "\n");
			flush.console();
		}
	}

	task_elapsed<-function(loop, msg=NULL)
	{
		re <- task.get(loop);

		if(!is.null(msg))
		{
			if (re$per!=0)
				sTime<-paste( "[", sprintf("%.2f", re$per), "%] ", "Elapsed:", re$elapse, " Left:", re$left, ". ")
			else
				sTime<-paste( "[ 0.00 %].");

			newmsg <- sub("$SYS_PROMPT$", sTime, msg, fixed=TRUE);
			cat( newmsg, "\n", sep="" );
			flush.console();
		}
	}

	task_stop<-function( msg=NULL )
	{
		task.pop();
		if (!is.null(msg))
		{
			cat( msg, "\n" );
			flush.console();
		}
	}

	set_value<-function( key, value)
	{
		assign( key, value, i_hash);
	}

	get_value<-function( key, def=NULL )
	{
		r<-NULL;
		try(r<-get( key, i_hash), TRUE);

		if (is.null(r) && !is.null(def))
			return(def);
			
		return (r);
	}


	return (
		list(reset         = reset, 
		    task_reset    = task.reset,
			seq_id        = get_seq_id, 
			task_start    = task_start, 
			task_stop     = task_stop, 
			task_elapsed  = task_elapsed,
			set_value     = set_value,
			get_value     = get_value,
			get_tmpfile   = get_tmpfile))
}
 
#sys<-Statgen.sys("abc");
#> sys$seq_id();
#[1] 1
#> sys$seq_id();
#[1] 2
#> sys$seq_id();
#[1] 3
#> sys$reset();
#> sys$seq_id();
#[1] 0

#--------------------------------------------------------------
# fGWAS.set_value/get_value()
#--------------------------------------------------------------
fGWAS.set_value <- function( key, value)
{
	if (is.null( fGWAS_sys ) )
		fGWAS.start();
	
	fGWAS_sys$set_value(key, value);
}

fGWAS.get_value <- function( key, def=NULL )
{
	if (is.null( fGWAS_sys ) )
		fGWAS.start();
	
	r<-NA;
	try( r <- fGWAS_sys$get_value(key, def), TRUE );
	return(r);
}


fGWAS.get_subfile<-function( baseFile, newExt, oldExt=".csv" )
{
	newFile <- sub( oldExt, newExt, baseFile );
	if ( baseFile == newFile )
		newFile <- paste( baseFile, newExt, sep="" );

	return( newFile );
}

#--------------------------------------------------------------
# Global system mamaner.
#--------------------------------------------------------------

fGWAS_sys <- NULL;

fGWAS.start <- function()
{
	fGWAS_sys <<- Statgen.sys("fGWAS");
	fGWAS_sys$reset();
	fGWAS_sys$set_value ("MaxIter", 2000);
	fGWAS_sys$set_value("RelRatio", 0.25);
	fGWAS_sys$set_value("ReduceOrder", 3);
	fGWAS_sys$set_value("McmcHint", 100);
}

fGWAS.start();

